/*********************************************************
*Copyright (C), 2017, Shanghai Eastsoft Microelectronics Co., Ltd.
*文件名:  uart.c
*作  者:  AE Team
*版  本:  V1.01
*日  期:  2021/05/05
*描  述:  UART模块程序（中断收发）
*备  注:  本软件仅供学习和演示使用，对用户直接引用代码所带来的风险或后果不承担任何法律责任。
**********************************************************/
#include "uart.h"

uint8_t g_rxbuf[255];           /* 接收数据缓存 */
uint8_t g_rx_length;            /* 接收数据长度 */
uint8_t g_txbuf[255];           /* 发送数据缓存 */
uint8_t g_tx_length;            /* 发送数据长度 */
uint8_t g_rx_time;              /* 字节间隔 */
uint8_t g_tx_time;              /* 字节间隔 */
volatile uint8_t g_rx_count;            /* 接收字节计数 */
volatile uint8_t g_tx_count;            /* 发送字节计数 */
volatile uint8_t g_rx_start_flag;       /* 接收起始标志 */
volatile uint8_t g_tx_start_flag;       /* 发送起始标志 */
volatile uint8_t g_rx_finish_flag;      /* 接收完成标志 */
volatile uint8_t g_tx_finish_flag;      /* 发送完成标志 */

/*********************************************************
函数名: void UART_Pins_Init(void)
描  述: 初始化UART通讯IO
输入值: 无
输出值: 无
返回值: 无
**********************************************************/
void UART_Pins_Init(void)
{
    GPIO_InitSettingType x;

    x.Signal = GPIO_Pin_Signal_Digital;
    x.Dir = GPIO_Direction_Output;
    x.Func = GPIO_Reuse_Func1;
    x.ODE = GPIO_ODE_Output_Disable;
    x.DS = GPIO_DS_Output_Strong;
    x.PUE = GPIO_PUE_Input_Disable;
    x.PDE = GPIO_PDE_Input_Disable;
    GPIO_Init(GPIO_Pin_A23, &x);        /* TXD - PA23 */

    x.Signal = GPIO_Pin_Signal_Digital;
    x.Func = GPIO_Reuse_Func1;
    x.Dir = GPIO_Direction_Input;
    x.PUE = GPIO_PUE_Input_Disable;
    x.PDE = GPIO_PDE_Input_Disable;
    GPIO_Init(GPIO_Pin_A22, &x);        /* RXD - PA22 */
}


/*********************************************************
函数名: void UARTInit(void)
描  述: UART初始化子程序
输入值: 无
输出值: 无
返回值: 无
**********************************************************/
void UARTInit(void)
{
    UART_InitStruType y;
    y.UART_ClockSet = UART_Clock_1;         /* 时钟选择：Pclk */
    y.UART_BaudRate = 115200;               /* 波特率:115200 */
    y.UART_StopBits = UART_StopBits_1;      /* 停止位：1 */
    y.UART_TxMode = UART_DataMode_8;        /* 发送数据格式：8位数据，无校验 */
    y.UART_TxPolar = UART_Polar_Normal;     /* 发送端口极性：正常 */
    y.UART_RxMode = UART_DataMode_8;        /* 接收数据格式：8位数据，无校验 */
    y.UART_RxPolar = UART_Polar_Normal;     /* 接收端口极性：正常 */
    UART_Init(UART0, &y);

    UART_TBIMConfig(UART0, UART_TBIM_Full);                      /* UART0发送缓冲区空中断模式: 全空中断 */
    UART_RBIMConfig(UART0, UART_TBIM_Byte);                      /* UART0接收缓冲区空中断模式: 字节满中断 */

    UART_ITConfig(UART0, UART_IT_RB, Enable);
	//UART_ClearITPendingBit(UART0, UART_FLAG_TC); /* 建议先清发送完成中断标志，防止出现误动作 */
    UART_ITConfig(UART0, UART_IT_TC, Enable);    /* 使能发送完成中断 */
    NVIC_Init(NVIC_UART0_IRQn, NVIC_Priority_1, Enable);

    UART_Pins_Init();
    UART0_TxEnable();
    UART0_RxEnable();
}

/*********************************************************
函数名: void UARTTxData(uint8_t *buf, uint8_t n)
描  述: UART发送数据整帧打包
输入值: buf—发送数据缓存
        n—发送数据个数
输出值: 无
返回值: 无
**********************************************************/
void UARTTxData(uint8_t *buf, uint8_t n)
{
    uint8_t i, check_sum;

    g_txbuf[0] = HEAD1;             /* 帧头0x55 */
    g_txbuf[1] = HEAD2;             /* 帧头0xAA */
    g_txbuf[2] = SLAVE_ADDR;        /* 设备地址 */
    memcpy(g_txbuf + 3, buf, n);

    check_sum = 0;
    g_tx_finish_flag = 0x1U;

    for (i = 0; i < n + 3; i++)
        check_sum ^= g_txbuf[i];    /* 计算包括帧头和地址在内的数据的异或校验和 */

    g_txbuf[n + 3] = check_sum;     /* 赋值校验和 */
    g_txbuf[n + 4] = TAIL1;         /* 帧尾0x5A */
    g_txbuf[n + 5] = TAIL2;         /* 帧尾0xA5 */

    g_tx_count = 0;
    g_tx_length = n + 6;

	UART_SendByte(UART0, g_txbuf[g_tx_count++]);
	
    while (g_tx_finish_flag == 0x1);
}

/*********************************************************
函数名: uint8_t UARTRxData(uint8_t *buf)
描  述: UART接收数据整帧解包
输入值: 无
输出值: buf—接收数据缓存
返回值: 正确解包返回数据帧字节长度
**********************************************************/
uint8_t UARTRxData(uint8_t *buf)
{
    uint8_t i;
    uint8_t check_sum = 0x0U;

    g_rx_finish_flag = 0x1U;
    g_rx_count  = 0x0U;
    g_rx_length = 0x0U;

    while (g_rx_finish_flag);

    if (g_rx_count < 6)                     /* 按通信协议，至少包含6个字节 */
    {
        memset(g_rxbuf, 0x00, sizeof(g_rxbuf));
        return 0;
    }
    else if ((g_rxbuf[0] != HEAD1) || (g_rxbuf[1] != HEAD2) || (g_rxbuf[2] != SLAVE_ADDR) ||
             (g_rxbuf[g_rx_count - 2] != TAIL1) || (g_rxbuf[g_rx_count - 1] != TAIL2))  /* 判断帧格式 */
    {
        memset(g_rxbuf, 0x00, sizeof(g_rxbuf));
        return 0;
    }
    else
    {
        for (i = 0; i < g_rx_count - 3; i++)
            check_sum ^= g_rxbuf[i];    /* 计算包括帧头和地址在内的数据的异或校验和 */

        if (g_rxbuf[g_rx_count - 3] != check_sum)    /* 判断校验和错误 */
        {
            memset(g_rxbuf, 0x00, sizeof(g_rxbuf));
            return 0;
        }
        else
        {
            memcpy(buf, g_rxbuf + 3, g_rx_count - 6);
            memset(g_rxbuf, 0x00, g_rx_count);
            g_rx_length = g_rx_count - 6;

            return g_rx_length;
        }
    }
}

